package com.redxun.knowledge.common;

import com.redxun.common.model.JPaasUser;
import com.redxun.common.tool.StringUtils;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.common.utils.SysUserUtil;
import com.redxun.feign.org.UserClient;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;

/**
 * 文件名: UserService
 * 作者: Zizg
 * 时间: 2022/11/29
 * 描述:
 * 修改人:
 * 修改时间:
 * 修改内容:
 */
@Component
@Slf4j
public class UserService {

    @Autowired
    private UserClient userClient;

    /**
     * 获取当前系统登录用户id
     *
     * @return
     */
    public String queryLoginUser() {
        String loginUserId = null;
        try {
            JPaasUser jPaasUser = SysUserUtil.getLoginUser();
            if (jPaasUser == null) {
                return null;
            }
            loginUserId = jPaasUser.getUserId();
        } catch (IOException e) {
            log.error(ExceptionUtil.getExceptionMessage(e));
        }

        return loginUserId;
    }

    /**
     * 获取当前系统登录用户姓名
     *
     * @return
     */
    public String queryLoginUserName() {
        String userId = queryLoginUser();
        String fullName = userClient.findByUserId(userId).getFullName();
        return fullName;
    }

    /**
     * 根据用户ID查询用户对象
     *
     * @param userId 用户ID
     * @return
     */
    public JPaasUser queryUser(String userId) {
        if (StringUtils.isEmpty(userId)) {
            return null;
        }
        return userClient.getUserById(userId);
    }
}
