
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.entity.dao.Banner;
import com.redxun.knowledge.entity.vo.BannerPcVo;
import com.redxun.knowledge.service.BannerServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/banner")
@Api(tags = "业务--首页banner表")
@ClassDefine(title = "业务--首页banner表", alias = "BannerController", path = "/api-knowledge/knowledge/banner", packages = "", packageName = "子系统名称")
public class BannerController extends BaseController<Banner> {

    @Autowired
    BannerServiceImpl bannerService;


    @Override
    public BaseService getBaseService() {
        return bannerService;
    }

    @Override
    public String getComment() {
        return "业务--首页banner表";
    }

    @ApiOperation(value = "Banner列表(分页后台)")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllBanner")
    public JsonPageResult findAllBanner(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = bannerService.findAllBanner(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }


    @ApiOperation(value = "Banner列表(无分页,前台)")
    @GetMapping("getAllList")
    public JsonPageResult getAllList() {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        List<BannerPcVo> bannerPcVoList = bannerService.getAllList();
        jsonResult.setData(bannerPcVoList);
        return jsonResult;
    }

    @ApiOperation(value = "删除banner")
    @GetMapping("deleteBanner")
    public JsonResult deleteBanner(@RequestParam("bannerId") String bannerId){
        bannerService.deleteBanner(bannerId);
        return JsonResult.Success("删除成功!");
    }

    @ApiOperation(value = "上线/下线")
    @GetMapping("updateStatus/{bannerId}/{status}")
    public JsonResult updateStatus(@PathVariable("bannerId") String bannerId,
                                   @PathVariable("status") Integer status){
        Banner banner = new Banner();
        banner.setPkId(bannerId);
        banner.setEnabled(status);
        banner.setUpdateTime(new Date());
        bannerService.updateById(banner);
        return JsonResult.Success();
    }

    @ApiOperation(value = "上升/下降")
    @GetMapping("/goUpOrDownUp/{bannerId}/{status}")
    public JsonResult goUpOrDownUp(@PathVariable("bannerId") String bannerId,
                                   @PathVariable("status") Boolean status){
        bannerService.goUpOrDownUp(bannerId,status);
        if (status){
            return JsonResult.Success("上升成功!");
        } else {
            return JsonResult.Success("下降成功!");
        }

    }

}

