
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dto.knowledgeDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.entity.vo.KnowledgeVo;
import com.redxun.knowledge.service.KnowledgeServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import oracle.jdbc.proxy.annotation.Post;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge")
@Api(tags = "业务--知识表")
@ClassDefine(title = "业务--知识表", alias = "KnowledgeController", path = "/knowledge/knowledge", packages = "", packageName = "子系统名称")
public class KnowledgeController extends BaseController<Knowledge> {

    @Autowired
    KnowledgeServiceImpl knowledgeService;


    @Override
    public BaseService getBaseService() {
        return knowledgeService;
    }

    @Override
    public String getComment() {
        return "业务--知识表";
    }

    @ApiOperation(value = "根据分类id查询全部维基知识", httpMethod = "GET")
    @GetMapping("/findAllWiki/{categoryId}")
    public JsonResult findAllWiki(@PathVariable("categoryId") String categoryId) {
        JsonResult jsonResult = JsonResult.getSuccessResult("返回数据成功!");
        List<Knowledge> wikiList = knowledgeService.findAllWiki(categoryId);
        jsonResult.setData(wikiList);
        return jsonResult;
    }

    @ApiOperation(value = "新增知识", httpMethod = "POST")
    @PostMapping("create")
    public JsonResult create(@Validated @RequestBody knowledgeDto knowledgeDto) {
        JsonResult jsonResult = JsonResult.getSuccessResult("新增知识成功！");
        String pkId = knowledgeService.create(knowledgeDto);
        jsonResult.setData(pkId);
        return jsonResult;
    }

    @ApiOperation(value = "知识详情", httpMethod = "GET")
    @GetMapping("info")
    public JsonResult info(@RequestParam(value = "pkId") String pkId) {
        JsonResult jsonResult = JsonResult.getSuccessResult("获取知识详情成功！");
        jsonResult.setData(knowledgeService.info(pkId));
        return jsonResult;
    }

    @ApiOperation(value = "知识列表(分页后台)")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllKnowledge")
    public JsonPageResult findAllKnowledge(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeService.findAllKnowledge(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation(value = "删除知识")
    @GetMapping("deleteKnowledge")
    public JsonResult deleteKnowledge(@RequestParam("knowledgeId") String knowledgeId){
        Boolean result = knowledgeService.deleteKnowledge(knowledgeId);
        if (!result){
            return JsonResult.Fail("该知识下还有关联banner,删除失败");
        }
        return JsonResult.Success("删除成功!");
    }

    @ApiOperation(value = "知识审核")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllAuditKnowledge")
    public JsonPageResult findAllAuditKnowledge(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeService.findAllAuditKnowledge(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation(value = "工作成果&经验案例列表(分页前台)")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("getAllKnowledgeByCategoryId")
    public JsonPageResult getAllKnowledgeByCategoryId(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = knowledgeService.getAllKnowledgeByCategoryId(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }
}

