
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.knowledge.entity.dao.KnowledgeLabel;
import com.redxun.knowledge.entity.vo.KnowledgeLabelVo;
import com.redxun.knowledge.service.KnowledgeLabelServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/knowledgeLabel")
@Api(tags = "业务--知识标签表")
@ClassDefine(title = "业务--知识标签表", alias = "KnowledgeLabelController", path = "/api-knowledge/knowledge/knowledgeLabel", packages = "", packageName = "子系统名称")
public class KnowledgeLabelController extends BaseController<KnowledgeLabel> {

    @Autowired
    KnowledgeLabelServiceImpl knowledgeLabelService;


    @Override
    public BaseService getBaseService() {
        return knowledgeLabelService;
    }

    @Override
    public String getComment() {
        return "业务--知识标签表";
    }

    @ApiOperation(value = "新增/修改标签数据", httpMethod = "POST")
    @PostMapping("saveOrUpdateTags")
    public JsonResult saveOrUpdateTags(@RequestParam(value = "id", required = false) String id, @RequestParam(value = "name", required = true) String name, @RequestParam(value = "sort", required = true) Integer sort) {
        JsonResult jsonResult = JsonResult.getSuccessResult("请求成功!");
        try {
            KnowledgeLabel result = knowledgeLabelService.saveOrUpdateTags(id, name, sort);
            if (ObjectUtils.isEmpty(result)) {
                jsonResult.setSuccess(false);
            }
            jsonResult.setData(result);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    /**
     * 知识仓库首页-快捷标签列表
     *
     * @param tops 返回条数
     * @return
     */
    @ApiOperation(value = "知识仓库首页-快捷标签列表", httpMethod = "GET")
    @GetMapping("labels")
    public JsonResult labels(@RequestParam(value = "tops", required = false) Integer tops) {
        JsonResult jsonResult = JsonResult.getSuccessResult("返回数据成功!");

        if (tops == null) {
            tops = 24;
        }

        try {
            jsonResult.setData(knowledgeLabelService.labels(tops));
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    /**
     * 分页条件搜索标签管理列表
     *
     * @param queryData name	标签名称
     *                  createTimeBegin	开始时间
     *                  createTimeEnd	结束时间
     * @return
     */
    @ApiOperation(value = "分页条件搜索标签管理列表", httpMethod = "POST")
    @PostMapping("findAllTags")
    public JsonPageResult findAllTags(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage<KnowledgeLabelVo> page = knowledgeLabelService.findAllTags(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    /**
     * 调整知识标签位置
     *
     * @param labelId 标签ID
     * @param status  true 上升  false 下降
     * @return
     */
    @ApiOperation(value = "调整知识标签排序位置", httpMethod = "GET")
    @GetMapping("/exchangePosition/{labelId}/{status}")
    public JsonResult exchangePosition(@PathVariable("labelId") String labelId,
                                       @PathVariable("status") Boolean status) {
        knowledgeLabelService.exchangePosition(labelId, status);
        return JsonResult.Success("位置交换成功!");
    }

    /**
     * 删除知识标签
     *
     * @param labelId 标签ID
     * @return
     */
    @ApiOperation(value = "删除知识标签", httpMethod = "GET")
    @PostMapping("delLabel")
    public JsonResult delLabel(@RequestParam("labelId") String labelId) {
        JsonResult jsonResult = JsonResult.getSuccessResult("删除知识标签成功!");
        jsonResult.setData(knowledgeLabelService.delLabel(labelId));
        return jsonResult;
    }
}

