
package com.redxun.knowledge.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.annotation.ClassDefine;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.entity.JsonPageResult;
import com.redxun.common.base.entity.JsonResult;
import com.redxun.common.base.entity.QueryData;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.base.search.QueryFilterBuilder;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.utils.ExceptionUtil;
import com.redxun.common.utils.SysUserUtil;
import com.redxun.knowledge.entity.dao.Banner;
import com.redxun.knowledge.entity.dao.News;
import com.redxun.knowledge.entity.vo.NewsPcVo;
import com.redxun.knowledge.service.NewsServiceImpl;
import com.redxun.web.controller.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@RestController
@RequestMapping("/api-knowledge/knowledge/news")
@Api(tags = "业务--新闻表")
@ClassDefine(title = "业务--新闻表", alias = "NewsController", path = "/api-knowledge/knowledge/news", packages = "", packageName = "子系统名称")
public class NewsController extends BaseController<News> {

    @Autowired
    NewsServiceImpl newsService;

    @Autowired
    private RedisTemplate redisTemplate;


    @Override
    public BaseService getBaseService() {
        return newsService;
    }

    @Override
    public String getComment() {
        return "业务--新闻表";
    }

    @ApiOperation(value = "新闻列表(有分页,后台)")
    @ApiImplicitParam(name = "queryData", value = "查询条件")
    @PostMapping("findAllNews")
    public JsonPageResult findAllNews(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = newsService.findAllNews(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation(value = "删除新闻")
    @GetMapping("deleteNews")
    public JsonResult deleteBanner(@RequestParam("newsId") String newsId){
        newsService.delete(newsId);
        return JsonResult.Success("删除成功!");
    }

    @ApiOperation(value = "上线/下线")
    @GetMapping("updateStatus/{newsId}/{status}")
    public JsonResult updateStatus(@PathVariable("newsId") String newsId,
                                   @PathVariable("status") Integer status){
        News news = new News();
        news.setPkId(newsId);
        news.setEnabled(status);
        news.setUpdateTime(new Date());
        newsService.updateById(news);
        return JsonResult.Success();
    }

    @ApiOperation(value = "上升/下降")
    @GetMapping("/goUpOrDownUp/{newsId}/{status}")
    public JsonResult goUpOrDownUp(@PathVariable("newsId") String newsId,
                                   @PathVariable("status") Boolean status){
        newsService.goUpOrDownUp(newsId,status);
        return JsonResult.Success("位置交换成功!");
    }

    @ApiOperation(value = "新闻资讯(无分页,前台)")
    @GetMapping("getAllList")
    public JsonPageResult getAllList() {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        List<NewsPcVo> newsPcVoList = newsService.getAllList();
        jsonResult.setData(newsPcVoList);
        return jsonResult;
    }

    @ApiOperation(value = "前台新闻列表")
    @PostMapping("findAllPcNews")
    public JsonPageResult findAllPcNews(@RequestBody QueryData queryData) {
        JsonPageResult jsonResult = JsonPageResult.getSuccess("返回数据成功!");
        try {
            QueryFilter filter = QueryFilterBuilder.createQueryFilter(queryData);
            IPage page = newsService.findAllPcNews(filter);
            this.handlePage(page);
            jsonResult.setPageData(page);
        } catch (Exception e) {
            jsonResult.setSuccess(false);
            this.logger.error(ExceptionUtil.getExceptionMessage(e));
            jsonResult.setMessage(ExceptionUtil.getExceptionMessage(e));
        }
        return jsonResult;
    }

    @ApiOperation("前台新闻详情")
    @GetMapping("findByIdPcNews")
    public JsonResult getAllList(@RequestParam("newsId") String newsId,
                                 HttpSession httpSession) {
        JsonResult<News> newsJsonResult = this.get(newsId);
        // 统计浏览数量
        String loginUserId = "1001";
        List<String> newsIdAttr = (List<String>) httpSession.getServletContext().getAttribute(loginUserId);
        if (newsIdAttr == null){
            List<String> initNewsIds = new ArrayList<>();
            initNewsIds.add(newsId);
            httpSession.getServletContext().setAttribute(loginUserId,initNewsIds);
            redisTemplate.opsForValue().set(newsId,newsJsonResult.getData().getViews());
        }
        if (newsIdAttr != null || !(newsIdAttr.contains(newsId))){
            newsIdAttr.add(newsId);
            httpSession.getServletContext().setAttribute(loginUserId,newsIdAttr);
        }
        newsIdAttr.forEach(e -> {
            if (redisTemplate.hasKey(e)){
                // 获取该新闻浏览数量
                Integer views = (Integer) redisTemplate.opsForValue().get(e);
                redisTemplate.opsForValue().set(e,++views);
            } else {
                Integer views = newsJsonResult.getData().getViews();
                redisTemplate.opsForValue().set(e,++views);
            }
        });
        return newsJsonResult;
    }

}

