
package com.redxun.knowledge.service;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.tool.IdGenerator;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.knowledge.entity.dao.Knowledge;
import com.redxun.knowledge.entity.dao.KnowledgeApprove;
import com.redxun.knowledge.entity.vo.BannerAdminVo;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalListVo;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.mapper.KnowledgeApproveMapper;
import javafx.scene.layout.BackgroundRepeat;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;
import javax.annotation.Resource;

/**
* [业务--知识审批表]业务服务类
*/
@Service
public class KnowledgeApproveServiceImpl extends SuperServiceImpl<KnowledgeApproveMapper, KnowledgeApprove> implements BaseService<KnowledgeApprove> {

    @Resource
    private KnowledgeApproveMapper knowledgeApproveMapper;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Override
    public BaseDao<KnowledgeApprove> getRepository() {
        return knowledgeApproveMapper;
    }

    public IPage findAllKnowledgeApprove(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeApproveMapper.findAllKnowledgeApprove(filter.getPage(), filter.getParams(), params);
        // 查询出全部记录
        List<KnowledgeApprovalListVo> knowledgeApprovalListVos = page.getRecords();
        // 根据知识Id进行数据分组
        Map<String, List<KnowledgeApprovalListVo>> result =
                knowledgeApprovalListVos.stream().collect(Collectors.groupingBy(KnowledgeApprovalListVo::getKnowledgerId));
        // 结果集
        List<KnowledgeApprovalListVo> knowledgeApprovalListVoList = new ArrayList<>();
        // 遍历每组的数据
        Set<Map.Entry<String, List<KnowledgeApprovalListVo>>> entries = result.entrySet();
        for (Map.Entry<String, List<KnowledgeApprovalListVo>> entry : entries) {
            List<KnowledgeApprovalListVo> value = entry.getValue();
            for (int i = 0; i < value.size(); i++) {
                // 判断审批结果是否为null(如果审批结果没有null，证明审批已结束;如果等于0，证明审批没通过)
                if (value.get(i).getResult() == null || value.get(i).getResult() == 0) {
                    if (i != 0) {
                        // 如果不是第一条数据,取前一条数据的审批时间
                        Date approvalTime = value.get(i - 1).getApprovalTime();
                        value.get(i).setApprovalTime(approvalTime);
                    }
                    // 把该记录加到结果集中
                    knowledgeApprovalListVoList.add(value.get(i));
                    break;
                }
            }
        }
        // 放入知识分类信息
        knowledgeApprovalListVoList.forEach(e -> {
            String categoryId = e.getCategoryId();
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(categoryId);
            e.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVo);
        });
        // 重新组装分页数据
        page.setRecords(knowledgeApprovalListVoList);
        page.setTotal(knowledgeApprovalListVoList.size());
        return page;
    }
}
