
package com.redxun.knowledge.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.redxun.common.base.db.BaseDao;
import com.redxun.common.base.db.BaseService;
import com.redxun.common.base.db.PageHelper;
import com.redxun.common.base.search.QueryFilter;
import com.redxun.common.model.JPaasUser;
import com.redxun.common.service.impl.SuperServiceImpl;
import com.redxun.common.tool.IdGenerator;
import com.redxun.knowledge.common.UserService;
import com.redxun.knowledge.entity.dao.*;
import com.redxun.knowledge.entity.dto.knowledgeDto;
import com.redxun.knowledge.entity.vo.KnowledgeApprovalVo;
import com.redxun.knowledge.entity.vo.KnowledgeCategoryAdminVo;
import com.redxun.knowledge.entity.vo.KnowledgeVo;
import com.redxun.knowledge.mapper.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * [业务--知识表]业务服务类
 */
@Service
public class KnowledgeServiceImpl extends SuperServiceImpl<KnowledgeMapper, Knowledge> implements BaseService<Knowledge> {

    @Resource
    private KnowledgeMapper knowledgeMapper;

    @Resource
    private KnowledgeApprovalNodeMapper knowledgeApprovalNodeMapper;

    @Resource
    private KnowledgeApproveMapper knowledgeApproveMapper;

    @Resource
    private UserService userService;

    @Autowired
    private KnowledgeCategoryServiceImpl knowledgeCategoryService;

    @Autowired
    private BannerMapper bannerMapper;

    @Autowired
    private KnowledgeCategoryMapper knowledgeCategoryMapper;

    @Override
    public BaseDao<Knowledge> getRepository() {
        return knowledgeMapper;
    }

    /**
     * 查询全部维基知识
     */
    public List<Knowledge> findAllWiki(String categoryId) {
        QueryWrapper<Knowledge> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("CATEGORY_ID", categoryId);
        queryWrapper.eq("TYPE", 2);
        List<Knowledge> knowledgeWikiList = knowledgeMapper.selectList(queryWrapper);
        return knowledgeWikiList;
    }

    /**
     * 新增知识
     * @param knowledgeDto
     * @return
     */
    public String create(knowledgeDto knowledgeDto) {
        if (knowledgeDto != null) {
            Knowledge knowledge = new Knowledge();
            knowledge.setPkId(IdGenerator.getIdStr());
            knowledge.setTitles(knowledgeDto.getTitles());
            knowledge.setCategoryId(knowledgeDto.getCategoryId());
            knowledge.setType(knowledgeDto.getType());
            knowledge.setSummary(knowledgeDto.getSummary());
            knowledge.setAttachment(knowledgeDto.getAttachment());
            knowledge.setAttachmentType(knowledgeDto.getAttachmentType());
            knowledge.setContent(knowledgeDto.getContent());

            JPaasUser user = userService.queryUser(userService.queryLoginUser());
            if (user != null) {
                knowledge.setAuthor(user.getFullName());
            }

            insert(knowledge);

            // 从知识审批节点表拉取审批节点
            List<KnowledgeApprovalNode> approvalVoList = knowledgeApprovalNodeMapper.selectList(new QueryWrapper<KnowledgeApprovalNode>().eq("IS_DEL", 0).orderByAsc("IS_FINAL", "CREATE_TIME_"));
            if (ObjectUtils.isNotEmpty(approvalVoList)) {
                KnowledgeApprovalNode node;
                for (int index = 0; index < approvalVoList.size(); index++) {
                    node = approvalVoList.get(index);
                    if (node != null) {
                        KnowledgeApprove knowledgeApprove = new KnowledgeApprove();
                        knowledgeApprove.setPkId(IdGenerator.getIdStr());
                        knowledgeApprove.setName(node.getName());
                        knowledgeApprove.setKnowledgerId(knowledge.getPkId());
                        knowledgeApprove.setApproverId(node.getApproverId());
                        knowledgeApprove.setApproverName(node.getApproverName());
                        knowledgeApprove.setApprovalBatch(1);
                        knowledgeApprove.setSort(index + 1);
                        knowledgeApprove.setIsFinal(node.getIsFinal());
                        knowledgeApprove.setCreateBy(userService.queryLoginUser());
                        knowledgeApprove.setCreateTime(new Date());

                        knowledgeApproveMapper.insert(knowledgeApprove);
                    }
                }
            }

            return knowledge.getPkId();
        }

        return "";
    }

    /**
     * 数据回显
     * @param pkId
     * @return
     */
    public KnowledgeVo info(String pkId) {
        Knowledge knowledge = get(pkId);

        List<KnowledgeApprove> approvalVoList = knowledgeApproveMapper.selectList(new QueryWrapper<KnowledgeApprove>().eq("KNOWLEDGER_ID", pkId).orderByAsc("SORT"));
        List<KnowledgeApprovalVo> approvals = new ArrayList<>();
        // 审核节点
        if (ObjectUtils.isNotEmpty(approvalVoList)) {
            KnowledgeApprove approve = null;
            KnowledgeApprovalVo approvalVo = null;
            boolean isFirstNoneApprove = true;
            for (int index = 0; index < approvalVoList.size(); index++) {
                approve = approvalVoList.get(index);
                approvalVo = new KnowledgeApprovalVo();
                approvalVo.setPkId(approve.getPkId());
                approvalVo.setName(approve.getName());
                approvalVo.setApproverId(approve.getApproverId());
                approvalVo.setApproverName(approve.getApproverName());
                approvalVo.setActualApprover(approve.getActualApprover());
                approvalVo.setActualApproverName(approve.getActualApproverName());
                approvalVo.setApprovalTime(approve.getApprovalTime());
                approvalVo.setSort(approve.getSort());
                // 审核状态 ：提交审核 审核通过 审核驳回
                if (approve.getResult() == null && isFirstNoneApprove) {
                    isFirstNoneApprove = false;
                    approvalVo.setApprovalStatus("提交审核");
                } else {
                    if (approve.getResult() == null) {
                        approvalVo.setApprovalStatus("");
                    } else if (approve.getResult() == 0) {
                        approvalVo.setApprovalStatus("审核驳回");
                    } else if (approve.getResult() == 1) {
                        approvalVo.setApprovalStatus("审核通过");
                    }
                }
                approvals.add(approvalVo);
            }
        }

        KnowledgeVo knowledgeVo = new KnowledgeVo();
        knowledgeVo.setPkId(knowledge.getPkId());
        knowledgeVo.setCategoryId(knowledge.getCategoryId());
        knowledgeVo.setType(knowledge.getType());
        knowledgeVo.setTitles(knowledge.getTitles());
        knowledgeVo.setAuthor(knowledge.getAuthor());
        knowledgeVo.setContent(knowledge.getContent());
        knowledgeVo.setSummary(knowledge.getSummary());
        knowledgeVo.setAttachment(knowledge.getAttachment());
        knowledgeVo.setCreateTime(knowledge.getCreateTime());
        knowledgeVo.setApprovalStatus(knowledge.getApprovalStatus());
        knowledgeVo.setApprovals(approvals);
        knowledgeVo.setKnowledgeCategoryAdminVo(knowledgeCategoryService.findAllParentByCategoryId(knowledge.getCategoryId()));
        return knowledgeVo;
    }

    /**
     * 查看全部知识
     * @param filter
     * @return
     */
    public IPage findAllKnowledge(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeMapper.findAllKnowledge(filter.getPage(), filter.getParams(), params);
        List<KnowledgeVo> knowledgeVoList = page.getRecords();
        knowledgeVoList.forEach(e -> {
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(e.getCategoryId());
            e.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVo);
        });
        return page;
    }

    /**
     * 删除知识(若关联banner，则无法删除)
     * @param knowledgeId
     * @return
     */
    public Boolean deleteKnowledge(String knowledgeId) {
        // 查看是否关联着banner
        Integer count = bannerMapper.selectCount(new QueryWrapper<Banner>().eq("KNOWLEDGE_ID", knowledgeId));
        if (count > 0){
            return false;
        }
        // 把知识审批表中对应数据删除掉
        knowledgeApproveMapper.deleteByKnowledgeId(knowledgeId);
        knowledgeMapper.deleteKnowledge(knowledgeId);
        return true;
    }

    /**
     * 修改知识，如当前知识在审核状态，无法修改
     * @param entity
     * @return
     */
    @Override
    public int update(Knowledge entity) {
        Knowledge knowledge = this.get(entity.getPkId());
        if (knowledge.getApprovalStatus() == 3){
            throw new RuntimeException("当前知识正在审核中,无法修改");
        }
        entity.setApprovalStatus(1);
        // TODO: 2022/12/2  把审核表中批次加1
        return this.getRepository().updateById(entity);
    }

    /**
     * 查找待审核的知识(分页查询，超过99条只显示前99条记录)
     * @param filter
     * @return
     */
    public IPage findAllAuditKnowledge(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        // 只查询待节点审核的数据
        IPage page = knowledgeMapper.findAllAuditKnowledge(filter.getPage(), filter.getParams(), params);
        List<KnowledgeVo> knowledgeVoList = page.getRecords();
        // 查询其所属分类信息以及审核表第一节点的审核人信息
        knowledgeVoList.forEach(e -> {
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(e.getCategoryId());
            e.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVo);
            List<KnowledgeApprove> knowledgerList = knowledgeApproveMapper.selectList(
                    new QueryWrapper<KnowledgeApprove>().eq("KNOWLEDGER_ID", e.getPkId()).orderByAsc("SORT"));
            e.setApprovalsFirst(knowledgerList.get(0));
        });
        // 数据大于99条，取前99条数据返回
        if (page.getSize() > 99){
            List<KnowledgeVo> recordsPre = knowledgeVoList.stream().limit(99).collect(Collectors.toList());
            page.setRecords(recordsPre);
            page.setSize(99);
        }
        return page;
    }

    /**
     * 根据分类id查询工作成果列表和经验案例列表
     * @param filter
     * @return
     */
    public IPage getAllKnowledgeByCategoryId(QueryFilter filter) {
        Map<String, Object> params = PageHelper.constructParams(filter);
        IPage page = knowledgeMapper.getAllKnowledgeByCategoryId(filter.getPage(), filter.getParams(), params);
        List<KnowledgeVo> knowledgeVoList = page.getRecords();
        knowledgeVoList.forEach(e -> {
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = knowledgeCategoryService.findAllParentByCategoryId(e.getCategoryId());
            e.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVo);
        });
        List<KnowledgeVo> result = new ArrayList<>(knowledgeVoList);
        knowledgeVoList.forEach(e -> {
            //把一级分类下的分类的知识加入返回结果集
            KnowledgeCategory knowledgeCategory = knowledgeCategoryMapper.selectById(e.getCategoryId());
            KnowledgeCategoryAdminVo knowledgeCategoryAdminVo = new KnowledgeCategoryAdminVo();
            BeanUtils.copyProperties(knowledgeCategory,knowledgeCategoryAdminVo);
            List<KnowledgeCategoryAdminVo> childrens = knowledgeCategoryService.findChildrens(knowledgeCategoryAdminVo);
            childrens.forEach(children -> {
                List<Knowledge> knowledgeList = this.baseMapper.selectList(new QueryWrapper<Knowledge>().eq("CATEGORY_ID", children.getPkId()));
                List<KnowledgeVo> collect = knowledgeList.stream().map(h -> {
                    KnowledgeVo knowledgeVo = new KnowledgeVo();
                    BeanUtils.copyProperties(h, knowledgeVo);
                    return knowledgeVo;
                }).collect(Collectors.toList());

                collect.forEach(g -> {
                    KnowledgeCategoryAdminVo knowledgeCategoryAdminVos = knowledgeCategoryService.findAllParentByCategoryId(g.getCategoryId());
                    g.setKnowledgeCategoryAdminVo(knowledgeCategoryAdminVos);
                });

                result.addAll(collect);
            });
        });
        // 按照创建时间排序
        List<KnowledgeVo> collect = result.stream().
                sorted((t1, t2) -> t2.getCreateTime().compareTo(t1.getCreateTime())).collect(Collectors.toList());
        page.setRecords(collect);
        page.setTotal(collect.size());
        return page;
    }
}
